@@ -1,8 +1,7 @@
-#[macro_use]
-extern crate structopt;
-#[macro_use]
-extern crate clap;
+//! current `clap::arg_enum!` uses "non-ident macro path" feature, which was stabilized in
+//! Rust 1.31.0.
 
+use clap::arg_enum;
 use structopt::StructOpt;
 
 arg_enum! {
@@ -17,7 +16,7 @@ arg_enum! {
 #[derive(StructOpt, Debug)]
 struct Opt {
     /// Important argument.
-    #[structopt(raw(possible_values = "&Baz::variants()", case_insensitive = "true"))]
+    #[structopt(possible_values = &Baz::variants(), case_insensitive = true)]
     i: Baz,
 }
 
