@@ -8,7 +8,7 @@
 
 #![deny(missing_docs)]
 
-//! This crate defines the `StructOpt` trait and its custom derrive.
+//! This crate defines the `StructOpt` trait and its custom derive.
 //!
 //! ## Features
 //!
@@ -21,14 +21,22 @@
 //! structopt = { version = "0.2", default-features = false }
 //! ```
 //!
+//! Support for [`paw`](https://github.com/rust-cli/paw) (the
+//! `Command line argument paw-rser abstraction for main`) is disabled
+//! by default, but can be enabled in the `structopt` dependency
+//! with the feature `paw`:
+//!
+//! ```toml
+//! [dependencies]
+//! structopt = { version = "0.2", features = [ "paw" ] }
+//! paw = "1.0"
+//! ```
+//!
 //! ## How to `derive(StructOpt)`
 //!
-//! First, let's look at an example:
+//! First, let's look at the example:
 //!
 //! ```should_panic
-//! #[macro_use]
-//! extern crate structopt;
-//!
 //! use std::path::PathBuf;
 //! use structopt::StructOpt;
 //!
@@ -36,17 +44,30 @@
 //! #[structopt(name = "example", about = "An example of StructOpt usage.")]
 //! struct Opt {
 //!     /// Activate debug mode
-//!     #[structopt(short = "d", long = "debug")]
+//!     // short and long flags (-d, --debug) will be deduced from the field's name
+//!     #[structopt(short, long)]
 //!     debug: bool,
+//!
 //!     /// Set speed
-//!     #[structopt(short = "s", long = "speed", default_value = "42")]
+//!     // we don't want to name it "speed", need to look smart
+//!     #[structopt(short = "v", long = "velocity", default_value = "42")]
 //!     speed: f64,
+//!
 //!     /// Input file
 //!     #[structopt(parse(from_os_str))]
 //!     input: PathBuf,
+//!
 //!     /// Output file, stdout if not present
 //!     #[structopt(parse(from_os_str))]
 //!     output: Option<PathBuf>,
+//!
+//!     /// Where to write the output: to `stdout` or `file`
+//!     #[structopt(short)]
+//!     out_type: String,
+//!
+//!     /// File name: only required when `out` is set to `file`
+//!     #[structopt(name = "FILE", required_if("out_type", "file"))]
+//!     file_name: String,
 //! }
 //!
 //! fn main() {
@@ -59,57 +80,183 @@
 //! and the various `structopt` attributes are simply
 //! used for additional parameters.
 //!
-//! First, define a struct, whatever its name.  This structure will
-//! correspond to a `clap::App`.  Every method of `clap::App` in the
-//! form of `fn function_name(self, &str)` can be use through attributes
-//! placed on the struct. In our example above, the `about` attribute
-//! will become an `.about("An example of StructOpt usage.")` call on the
-//! generated `clap::App`. There are a few attributes that will default
-//! if not specified:
+//! First, define a struct, whatever its name. This structure
+//! corresponds to a `clap::App`, its fields correspond to `clap::Arg`
+//! (unless they're [subcommands](#subcommands)),
+//! and you can adjust these apps and args by `#[structopt(...)]` [attributes](#attributes).
 //!
-//!   - `name`: The binary name displayed in help messages. Defaults
-//!      to the crate name given by Cargo.
-//!   - `version`: Defaults to the crate version given by Cargo.
-//!   - `author`: Defaults to the crate author name given by Cargo.
-//!   - `about`: Defaults to the crate description given by Cargo.
+//! ## Attributes
 //!
-//! Methods from `clap::App` that don't take an `&str` can be called by
-//! wrapping them in `raw()`, e.g. to activate colored help text:
+//! `#[structopt(...)]` attributes fall into two categories:
+//! - `structopt`'s own [magical methods](#magical-methods).
+//!
+//!    They are used by `structopt` itself. They come mostly in
+//!    `attr = ["whatever"]` form, but some `attr(args...)` also exist.
+//!
+//! - [`raw` attributes](#raw-methods).
+//!
+//!     They represent explicit `clap::Arg/App` method calls.
+//!     They are what used to be explicit `#[structopt(raw(...))]` attrs in pre-0.3 `structopt`
+//!
+//! Every `structopt attribute` looks like comma-separated sequence of methods:
+//! ```rust,ignore
+//! #[structopt(
+//!     short, // method with no arguments - always magical
+//!     long = "--long-option", // method with one argument
+//!     required_if("out", "file"), // method with one and more args
+//!     parse(from_os_str = path::to::parser) // some magical methods have their own syntax
+//! )]
+//! ```
+//!
+//! `#[structopt(...)]` attributes can be placed on top of `struct`, `enum`,
+//! `struct` field or `enum` variant. Attributes on top of `struct` or `enum`
+//! represent `clap::App` method calls, field or variant attributes correspond
+//! to `clap::Arg` method calls.
+//!
+//! In other words, the `Opt` struct from the example above
+//! will be turned into this (*details omitted*):
 //!
 //! ```
-//! #[macro_use]
-//! extern crate structopt;
+//! # use structopt::clap::{Arg, App};
+//! App::new("example")
+//!     .version("0.2.0")
+//!     .about("An example of StructOpt usage.")
+//! .arg(Arg::with_name("debug")
+//!     .help("Activate debug mode")
+//!     .short("debug")
+//!     .long("debug"))
+//! .arg(Arg::with_name("speed")
+//!     .help("Set speed")
+//!     .short("s")
+//!     .long("speed")
+//!     .default_value("42"))
+//! // and so on
+//! # ;
+//! ```
 //!
-//! use structopt::StructOpt;
+//! ## Raw methods
 //!
-//! #[derive(StructOpt, Debug)]
-//! #[structopt(raw(setting = "structopt::clap::AppSettings::ColoredHelp"))]
-//! struct Opt {
-//!     #[structopt(short = "s")]
-//!     speed: bool,
-//!     #[structopt(short = "d")]
-//!     debug: bool,
+//! They are the reason why `structopt` is so flexible.
+//!
+//! Each and every method from `clap::App` and `clap::Arg` can be used directly -
+//! just `#[structopt(method_name = single_arg)]` or `#[structopt(method_name(arg1, arg2))]`
+//! and it just works. As long as `method_name` is not one of the magical methods -
+//! it's just a method call.
+//!
+//! **Note:** the `Arg::raw` method is allowed only with `true` or `false` literals.
+//!
+//! ## Magical methods
+//!
+//! They are the reason why `structopt` is so easy to use and convenient in most cases.
+//! Many of them have defaults, some of them get used even if not mentioned.
+//!
+//! Methods may be used on "top level" (on top of a `struct`, `enum` or `enum` variant)
+//! and/or on "field-level" (on top of a `struct` field or *inside* of an enum variant).
+//! Top level (non-magical) methods correspond to `App::method` calls, field-level methods
+//! are `Arg::method` calls.
+//!
+//! ```ignore
+//! #[structopt(top_level)]
+//! struct Foo {
+//!     #[structopt(field_level)]
+//!     field: u32
+//! }
+//!
+//! #[structopt(top_level)]
+//! enum Bar {
+//!     #[structopt(top_level)]
+//!     Pineapple {
+//!         #[structopt(field_level)]
+//!         chocolate: String
+//!     },
+//!
+//!     #[structopt(top_level)]
+//!     Orange,
 //! }
-//! # fn main() {}
 //! ```
 //!
-//! Then, each field of the struct not marked as a subcommand corresponds
-//! to a `clap::Arg`. As with the struct attributes, every method of
-//! `clap::Arg` in the form of `fn function_name(self, &str)` can be used
-//! through specifying it as an attribute. The `name` attribute can be used
-//! to customize the `Arg::with_name()` call (defaults to the field name in
-//! kebab-case).
-//! For functions that do not take a `&str` as argument, the attribute can be
-//! wrapped in `raw()`, e. g. `raw(aliases = r#"&["alias"]"#, next_line_help = "true")`.
+//! - `name`: `[name = "name"]`
+//!   - On top level: `App::new("name")`.
 //!
-//! The type of the field gives the kind of argument:
+//!     The binary name displayed in help messages. Defaults to the crate name given by Cargo.
 //!
-//! Type                 | Effect                                            | Added method call to `clap::Arg`
-//! ---------------------|---------------------------------------------------|--------------------------------------
-//! `bool`               | `true` if the flag is present                     | `.takes_value(false).multiple(false)`
-//! `Option<T: FromStr>` | optional positional argument or option            | `.takes_value(true).multiple(false)`
-//! `Vec<T: FromStr>`    | list of options or the other positional arguments | `.takes_value(true).multiple(true)`
-//! `T: FromStr`         | required option or positional argument            | `.takes_value(true).multiple(false).required(!has_default)`
+//!   - On field-level: `Arg::with_name("name")`.
+//!
+//!     The name for the argument the field stands for, this name appears in help messages.
+//!     Defaults to a name, deduced from a field, see also [`rename_all`](#specifying-argument-types).
+//!
+//! - `version`: `[version = "version"]`
+//!
+//!     Usable only on top level: `App::version("version" or env!(CARGO_PKG_VERSION))`.
+//!
+//!     The version displayed in help messages.
+//!     Defaults to the crate version given by Cargo. If `CARGO_PKG_VERSION` is not
+//!     set no `.version()` calls will be generated unless requested.
+//!
+//! - `no_version`: `no_version`
+//!
+//!     Usable only on top level. Prevents default `App::version` call, i.e
+//!     when no `version = "version"` mentioned.
+//!
+//! - `author`: `author [= "author"]`
+//!
+//!     Usable only on top level: `App::author("author" or env!(CARGO_PKG_AUTHOR))`.
+//!
+//!     Author/maintainer of the binary, this name appears in help messages.
+//!     Defaults to the crate author given by cargo, but only when `author` explicitly mentioned.
+//!
+//! - `about`: `about [= "about"]`
+//!
+//!     Usable only on top level: `App::about("about" or env!(CARGO_PKG_DESCRIPTION))`.
+//!
+//!     Short description of the binary, appears in help messages.
+//!     Defaults to the crate description given by cargo,
+//!     but only when `about` explicitly mentioned.
+//!
+//! - [`short`](#specifying-argument-types): `short [= "short-opt-name"]`
+//!
+//!     Usable only on field-level.
+//!
+//! - [`long`](#specifying-argument-types): `long [= "long-opt-name"]`
+//!
+//!     Usable only on field-level.
+//!
+//! - [`rename_all`](#specifying-argument-types): [`rename_all = "kebab"/"snake"/"screaming-snake"/"camel"/"pascal"/"verbatim"]`
+//!
+//!     Usable only on top level
+//!
+//! - [`parse`](#custom-string-parsers): `parse(type [= path::to::parser::fn])`
+//!
+//!     Usable only on field-level.
+//!
+//! - [`skip`](#skipping-fields): `skip`
+//!
+//!     Usable only on field-level.
+//!
+//! - [`flatten`](#flattening): `flatten`
+//!
+//!     Usable only on field-level.
+//!
+//! - [`subcommand`](#subcommands): `subcommand`
+//!
+//!     Usable only on field-level.
+//!
+//! ## Type magic
+//!
+//! One of major things that makes `structopt` so awesome is it's type magic.
+//! Do you want optional positional argument? Use `Option<T>`! Or perhaps optional argument
+//! that optionally takes value (`[--opt=[val]]`)? Use `Option<Option<T>>`!
+//!
+//! Here is the table of types and `clap` methods they correspond to:
+//!
+//! Type                         | Effect                                            | Added method call to `clap::Arg`
+//! -----------------------------|---------------------------------------------------|--------------------------------------
+//! `bool`                       | `true` if the flag is present                     | `.takes_value(false).multiple(false)`
+//! `Option<T: FromStr>`         | optional positional argument or option            | `.takes_value(true).multiple(false)`
+//! `Option<Option<T: FromStr>>` | optional option with optional value               | `.takes_value(true).multiple(false).min_values(0).max_values(1)`
+//! `Vec<T: FromStr>`            | list of options or the other positional arguments | `.takes_value(true).multiple(true)`
+//! `Option<Vec<T: FromStr>`     | optional list of options                          | `.takes_values(true).multiple(true).min_values(0)`
+//! `T: FromStr`                 | required option or positional argument            | `.takes_value(true).multiple(false).required(!has_default)`
 //!
 //! The `FromStr` trait is used to convert the argument to the given
 //! type, and the `Arg::validator` method is set to a method using
@@ -158,10 +305,12 @@
 //! If an argument is renamed using `name = $NAME` any following call to
 //! `short` or `long` will use the new name.
 //!
-//! ```
-//! #[macro_use]
-//! extern crate structopt;
+//! **Attention**: If these arguments are used without an explicit name
+//! the resulting flag is going to be renamed using `kebab-case` if the
+//! `rename_all` attribute was not specified previously. The same is true
+//! for subcommands with implicit naming through the related data structure.
 //!
+//! ```
 //! use structopt::StructOpt;
 //!
 //! #[derive(StructOpt)]
@@ -189,6 +338,12 @@
 //!     /// This option is positional, meaning it is the first unadorned string
 //!     /// you provide (multiple others could follow).
 //!     my_positional: String,
+//!
+//!     /// This option is skipped and will be filled with the default value
+//!     /// for its type (in this case 0).
+//!     #[structopt(skip)]
+//!     skipped: u32,
+//!
 //! }
 //!
 //! # fn main() {
@@ -205,12 +360,12 @@
 //! they can also be specified using doc comments. For example:
 //!
 //! ```
-//! # #[macro_use] extern crate structopt;
+//! # use structopt::StructOpt;
+//!
 //! #[derive(StructOpt)]
-//! #[structopt(name = "foo")]
 //! /// The help message that will be displayed when passing `--help`.
 //! struct Foo {
-//!   #[structopt(short = "b")]
+//!   #[structopt(short)]
 //!   /// The description for the arg that will be displayed when passing `--help`.
 //!   bar: String
 //! }
@@ -238,12 +393,12 @@
 //! the long descriptive message.
 //!
 //! ```
-//! # #[macro_use] extern crate structopt;
+//! # use structopt::StructOpt;
+//!
 //! #[derive(StructOpt)]
-//! #[structopt(name = "foo")]
 //! /// The help message that will be displayed when passing `--help`.
 //! struct Foo {
-//!   #[structopt(short = "b")]
+//!   #[structopt(short)]
 //!   /// Only this summary is visible when passing `-h`.
 //!   ///
 //!   /// But the whole comment will be displayed when passing `--help`.
@@ -254,6 +409,54 @@
 //! # fn main() {}
 //! ```
 //!
+//! ## Environment Variable Fallback
+//!
+//! It is possible to specify an environment variable fallback option for an arguments
+//! so that its value is taken from the specified environment variable if not
+//! given through the command-line:
+//!
+//! ```
+//! # use structopt::StructOpt;
+//!
+//! #[derive(StructOpt)]
+//! struct Foo {
+//!   #[structopt(short, long, env = "PARAMETER_VALUE")]
+//!   param: String
+//! }
+//! # fn main() {}
+//! ```
+//!
+//! By default, values from the environment are shown in the help output (i.e. when invoking
+//! `--help`):
+//!
+//! ```shell
+//! $ cargo run -- --help
+//! ...
+//! OPTIONS:
+//!   -p, --param <param>     [env: PARAMETER_VALUE=env_value]
+//! ```
+//!
+//! In some cases this may be undesirable, for example when being used for passing
+//! credentials or secret tokens. In those cases you can use `hide_env_values` to avoid
+//! having strucopt emit the actual secret values:
+//! ```
+//! # use structopt::StructOpt;
+//!
+//! #[derive(StructOpt)]
+//! struct Foo {
+//!   #[structopt(long = "secret", env = "SECRET_VALUE", hide_env_values = true)]
+//!   param: String
+//! }
+//! # fn main() {}
+//! ```
+//!
+//! ## Skipping fields
+//!
+//! Sometimes you may want to add a field to your `Opt` struct that is not
+//! a command line option and `clap` should know nothing about it. You can ask
+//! `structopt` to skip the field entirely via `#[structopt(skip)]`. Note that
+//! the field type has to implement `std::default::Default` then.
+//!
 //! ## Subcommands
 //!
 //! Some applications, especially large ones, split their functionality
@@ -265,33 +468,31 @@
 //! `clap` has this functionality, and `structopt` supports it through enums:
 //!
 //! ```
-//! # #[macro_use] extern crate structopt;
+//! # use structopt::StructOpt;
+//!
 //! # use std::path::PathBuf;
 //! #[derive(StructOpt)]
-//! #[structopt(name = "git", about = "the stupid content tracker")]
+//! #[structopt(about = "the stupid content tracker")]
 //! enum Git {
-//!     #[structopt(name = "add")]
 //!     Add {
-//!         #[structopt(short = "i")]
+//!         #[structopt(short)]
 //!         interactive: bool,
-//!         #[structopt(short = "p")]
+//!         #[structopt(short)]
 //!         patch: bool,
 //!         #[structopt(parse(from_os_str))]
 //!         files: Vec<PathBuf>
 //!     },
-//!     #[structopt(name = "fetch")]
 //!     Fetch {
-//!         #[structopt(long = "dry-run")]
+//!         #[structopt(long)]
 //!         dry_run: bool,
-//!         #[structopt(long = "all")]
+//!         #[structopt(long)]
 //!         all: bool,
 //!         repository: Option<String>
 //!     },
-//!     #[structopt(name = "commit")]
 //!     Commit {
-//!         #[structopt(short = "m")]
+//!         #[structopt(short)]
 //!         message: Option<String>,
-//!         #[structopt(short = "a")]
+//!         #[structopt(short)]
 //!         all: bool
 //!     }
 //! }
@@ -306,14 +507,12 @@
 //! need to apply to all subcommands, as well as nested subcommands:
 //!
 //! ```
-//! # #[macro_use] extern crate structopt;
+//! # use structopt::StructOpt;
 //! # fn main() {}
 //! #[derive(StructOpt)]
-//! #[structopt(name = "make-cookie")]
 //! struct MakeCookie {
 //!     #[structopt(name = "supervisor", default_value = "Puck", long = "supervisor")]
 //!     supervising_faerie: String,
-//!     #[structopt(name = "tree")]
 //!     /// The faerie tree this cookie is being made in.
 //!     tree: Option<String>,
 //!     #[structopt(subcommand)]  // Note that we mark a field as a subcommand
@@ -322,27 +521,24 @@
 //!
 //! #[derive(StructOpt)]
 //! enum Command {
-//!     #[structopt(name = "pound")]
 //!     /// Pound acorns into flour for cookie dough.
 //!     Pound {
 //!         acorns: u32
 //!     },
-//!     #[structopt(name = "sparkle")]
 //!     /// Add magical sparkles -- the secret ingredient!
 //!     Sparkle {
-//!         #[structopt(short = "m", parse(from_occurrences))]
+//!         #[structopt(short, parse(from_occurrences))]
 //!         magicality: u64,
-//!         #[structopt(short = "c")]
+//!         #[structopt(short)]
 //!         color: String
 //!     },
-//!     #[structopt(name = "finish")]
 //!     Finish(Finish),
 //! }
 //!
 //! // Subcommand can also be externalized by using a 1-uple enum variant
 //! #[derive(StructOpt)]
 //! struct Finish {
-//!     #[structopt(short = "t")]
+//!     #[structopt(short)]
 //!     time: u32,
 //!     #[structopt(subcommand)]  // Note that we mark a field as a subcommand
 //!     finish_type: FinishType
@@ -351,11 +547,9 @@
 //! // subsubcommand!
 //! #[derive(StructOpt)]
 //! enum FinishType {
-//!     #[structopt(name = "glaze")]
 //!     Glaze {
 //!         applications: u32
 //!     },
-//!     #[structopt(name = "powder")]
 //!     Powder {
 //!         flavor: String,
 //!         dips: u32
@@ -377,10 +571,9 @@
 //! A nested subcommand can be marked optional:
 //!
 //! ```
-//! # #[macro_use] extern crate structopt;
+//! # use structopt::StructOpt;
 //! # fn main() {}
 //! #[derive(StructOpt)]
-//! #[structopt(name = "foo")]
 //! struct Foo {
 //!     file: String,
 //!     #[structopt(subcommand)]
@@ -402,12 +595,12 @@
 //! a field as `flatten` and give it another type that derives `StructOpt`:
 //!
 //! ```
-//! # #[macro_use] extern crate structopt;
 //! # use structopt::StructOpt;
 //! # fn main() {}
 //! #[derive(StructOpt)]
 //! struct Cmdline {
-//!     #[structopt(short = "v", help = "switch on verbosity")]
+//!     /// switch on verbosity
+//!     #[structopt(short)]
 //!     verbose: bool,
 //!     #[structopt(flatten)]
 //!     daemon_opts: DaemonOpts,
@@ -415,9 +608,11 @@
 //!
 //! #[derive(StructOpt)]
 //! struct DaemonOpts {
-//!     #[structopt(short = "u", help = "daemon user")]
+//!     /// daemon user
+//!     #[structopt(short)]
 //!     user: String,
-//!     #[structopt(short = "g", help = "daemon group")]
+//!     /// daemon group
+//!     #[structopt(short)]
 //!     group: String,
 //! }
 //! ```
@@ -436,7 +631,7 @@
 //! provide a custom string parser using `parse(...)` like this:
 //!
 //! ```
-//! # #[macro_use] extern crate structopt;
+//! # use structopt::StructOpt;
 //! # fn main() {}
 //! use std::num::ParseIntError;
 //! use std::path::PathBuf;
@@ -447,9 +642,9 @@
 //!
 //! #[derive(StructOpt)]
 //! struct HexReader {
-//!     #[structopt(short = "n", parse(try_from_str = "parse_hex"))]
+//!     #[structopt(short, parse(try_from_str = parse_hex))]
 //!     number: u32,
-//!     #[structopt(short = "o", parse(from_os_str))]
+//!     #[structopt(short, parse(from_os_str))]
 //!     output: PathBuf,
 //! }
 //! ```
@@ -483,21 +678,13 @@
 //! once to validate, and once to parse. Hence, make sure the function is
 //! side-effect-free.
 
-extern crate clap as _clap;
-
-#[allow(unused_imports)]
-#[macro_use]
-extern crate structopt_derive;
-
 #[doc(hidden)]
 pub use structopt_derive::*;
 
 use std::ffi::OsString;
 
 /// Re-export of clap
-pub mod clap {
-    pub use _clap::*;
-}
+pub use clap;
 
 /// A struct that is converted from command line arguments.
 pub trait StructOpt {
@@ -506,7 +693,7 @@ pub trait StructOpt {
 
     /// Creates the struct from `clap::ArgMatches`.  It cannot fail
     /// with a parameter generated by `clap` by construction.
-    fn from_clap(&clap::ArgMatches) -> Self;
+    fn from_clap(matches: &clap::ArgMatches<'_>) -> Self;
 
     /// Gets the struct from the command line arguments.  Print the
     /// error message and quit the program in case of failure.
